<?php

namespace HCM\Geral\Logic\QuadroVagas;

use HCM\Util\{ Repositories, Util };

class Verificador
{
    private $tipoEstruturaControleVagas;
    private $tipoEstruturaGerencial;
    private $tipoValidacaoVagas;
    private $quantidadeLimiteVagas;
    private $nomeEstrutura;
    private $nomeOcupacao;
    
    const RESPONSE_STATUS_WARNING = 'A';
    const RESPONSE_STATUS_BLOCKED = 'I';

    public function __construct()
    {
        $this->entityManager = Util::getEntityManager();
        $this->hcmParameters = Util::getHCMParameters();

        $this->paramOrg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(
            ['nrorg' => $this->hcmParameters->getNrorgTrabalho()]);
            
        if (!empty($this->paramOrg)) {
            $this->tipoEstruturaControleVagas = $this->paramOrg->getNrtpestrutqvagas();
            $this->tipoEstruturaGerencial = $this->paramOrg->getNrtpestrutgeren();
            $this->tipoValidacaoVagas = $this->paramOrg->getIdvalidqvagas();   
        }
    }

    public function getVerificacao(array $movimentacoes): array
    {
        if (empty($this->tipoEstruturaControleVagas) || empty($this->tipoValidacaoVagas)) {
            return [
                'hasLimiteQuadroVagas' => false
            ];
        }

        foreach ($movimentacoes as $movimentacao) {
            
            if ($this->admissaoExcedeQuadroVagas($movimentacao)) {
                return [
                    'hasLimiteQuadroVagas' => true,
                    'tipoValidacao' => $this->tipoValidacaoVagas,
                    'mensagemVerificacao' => $this->getMensagemValidacaoVagas()
                ];
            }
        }
        
        return [
            'hasLimiteQuadroVagas' => false,
            'tipoValidacao' => $this->tipoValidacaoVagas,
            'mensagemVerificacao' => $this->getMensagemValidacaoVagas()
        ];
    }

    private function admissaoExcedeQuadroVagas(array $movimentacao): bool
    {
        if ($movimentacao['NRESTRUTURA'] == null) {
            return false;
        }

        $this->nomeOcupacao = $movimentacao['NMOCUPACAO'];
        $this->nomeEscala = !empty($movimentacao['NMESCALATRABH']) ? $movimentacao['NMESCALATRABH'] : '';
     
        
        if ($this->limiteMovimentacaoExcedido($movimentacao) || $this->limiteEstruturasSuperioresExcedido($movimentacao)) {
            return true;
        }

        return false;
    }

    private function limiteMovimentacaoExcedido(array $movimentacao): bool
    {
        
        $temMovimentacaoComVagasLimitadas = $this->isTipoEstruturaIgualControleVagas($movimentacao['TIPOESTRUTURA']) &&
            $this->excedeLimiteVagas($movimentacao['NRESTRUTURA'], $movimentacao['NROCUPACAO'], $movimentacao['NRESCALATRABM']);

        if ($temMovimentacaoComVagasLimitadas) {
            $this->nomeEstrutura = $movimentacao['NMESTRUTURA'];

            return true;
        }

        return false;
    }

    private function limiteEstruturasSuperioresExcedido(array $movimentacao): bool
    {
        if (!$this->isTipoEstruturaGerencial($movimentacao['TIPOESTRUTURA'])) {
            return false;
        }
        
        $estruturasSuperiores = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getEstruturasSuperiores($this->hcmParameters->getNrorgTrabalho(),
            $movimentacao['NRESTRUTURA']);
        
        foreach ($estruturasSuperiores as $estruturaSuperior) {
            if ($this->excedeLimiteVagas($estruturaSuperior['NRESTRUTURAM'], $movimentacao['NROCUPACAO'], $movimentacao['NRESCALATRABM'])) {
                $this->nomeEstrutura = $estruturaSuperior['NMESTRUTURAM'];

                return true;
            }
        }

        return false;
    }

    private function isTipoEstruturaIgualControleVagas(int $tipoEstrutura): bool
    {
        return $this->tipoEstruturaControleVagas == $tipoEstrutura;
    }
    
    private function isTipoEstruturaGerencial(int $tipoEstrutura): bool
    {
        return $this->tipoEstruturaGerencial == $tipoEstrutura;
    }

    private function excedeLimiteVagas(int $nrEstruturam, int $nrOcupacao, $nrEscalatrabm = null): bool
    {
        $quadrosVagas = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQuadrosAtivos(
            $this->hcmParameters->getNrorgTrabalho(), $nrEstruturam, $nrOcupacao, $nrEscalatrabm);
        
        if (empty($quadrosVagas)) {
            return false;
        }
        $this->quantidadeLimiteVagas = $this->getQuantidadeLimiteQuadrosAtivos($quadrosVagas);
        $quantidadeVinculosAtivos = $this->entityManager->getRepository(Repositories::GPE_QUADROVAGAS)->getQntVinculosAtivosOcupEstrutHierarq(
            $this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtCompetencia(), $nrOcupacao, $nrEstruturam, $nrEscalatrabm);

        if ($quantidadeVinculosAtivos >= $this->quantidadeLimiteVagas) {
            return true;
        }

        return false;
    }
    
    private function getQuantidadeLimiteQuadrosAtivos(array $quadrosVagas): int {
        $totalVagas = 0;
        foreach ($quadrosVagas as $quadroVagas) {
            $totalVagas += $quadroVagas['QTDLIMITE'];
        }
        
        return $totalVagas;
    }

    private function getMensagemValidacaoVagas(): string
    {
        $mensagemValidacao = [
            self::RESPONSE_STATUS_WARNING => "A admissão deste vínculo ultrapassará a quantidade limite definida de vagas (Limite: {$this->quantidadeLimiteVagas} vínculos) para a ocupação '{$this->nomeOcupacao}' dentro da estrutura '{$this->nomeEstrutura}' e com a escala '{$this->nomeEscala}'. Deseja prosseguir?",
            self::RESPONSE_STATUS_BLOCKED => "Operação bloqueada. A admissão deste vínculo ultrapassará a quantidade limite definida de vagas (Limite: {$this->quantidadeLimiteVagas} vínculos) para a ocupação '{$this->nomeOcupacao}' dentro da estrutura '{$this->nomeEstrutura}' e com a escala '{$this->nomeEscala}'."
        ];

        return $mensagemValidacao[$this->tipoValidacaoVagas];
    }
}